function [R,SIM]=gibbs_MC_LDA8(SIM, M, H)
%function [lpD,R]=gibbs_MC_LDA8(SIM, M, H)
% Input:
%   M:   List of model complexity: [Topics, Clusters] [Ny, Nz]
%   H:   List of hyperparams [alpha, beta, gamma]
%   SIM.: Data structure with options.
%       DETAILS: Verbosity level.
%       Ny: Number of topics
%       Nz: Number of clusters.
%
% Output
%   R:   Results structure.
%   SIM: Output information structure.

DETAILS = SIM.DETAILS;
if(SIM.RESET_RAND)
    %randn('state',SIM.RESET_RAND);
    %rand('state',SIM.RESET_RAND);
    defaultStream = RandStream.getDefaultStream;
    defaultStream.reset;
    fprintf(1,'Resetting Random Stream: First # %f/%f\n',rand,randn);
end

format compact;

TrueModel.T = SIM.T;
TrueModel.INFER_HMM    = SIM.INFER_HMM;
TrueModel.INIT_TRUE    = SIM.INIT_TRUE;
TrueModel.REGULAR_LDA  = SIM.REGULAR_LDA;
TrueModel.REGULAR_HMM  = SIM.REGULAR_HMM;
REGULAR_LDA = SIM.REGULAR_LDA;
REGULAR_HMM = SIM.REGULAR_HMM;
INFER_HMM = SIM.INFER_HMM;
INIT_TRUE = SIM.INIT_TRUE;


 
% Simulation Setup
if(SIM.REGULAR_LDA), NEMD = 4; else NEMD = 5; end
if(SIM.REGULAR_HMM), NEMD = 4; else NEMD = 5; end
Nm = SIM.Nm;
Ns = SIM.Ns;
LAG= SIM.LAG;
BURN=SIM.BURN;
Nest=Ns/LAG;
SIM.Nest=Nest;
if(nargin==1)
    Ny = SIM.Ny;
    Nz = SIM.Nz;
    fprintf(1,'Accepting model as SIM: Ny: %d, Nz: %d\n', Ny, Nz);
else    
    Ny = M(1);
    Nz = M(2);
    fprintf(1,'Accepting model as arg: Ny: %d, Nz: %d\n', Ny, Nz);
end
if(nargin<3)
    % True Model Setup
    if(isfield(SIM,'beta'))
        TrueModel.alpha = SIM.alpha; 
        TrueModel.beta =  SIM.beta; 
        TrueModel.gamma = SIM.gamma;
    else
        TrueModel.alpha = 1; 
        TrueModel.beta =  1; 
        TrueModel.gamma = 1;
    end
else
    TrueModel.alpha = H(1);
    TrueModel.beta  = H(2);
    TrueModel.gamma = H(3);
end

T  = SIM.T;
TrueModel.Ny = Ny;
TrueModel.Nz = Nz;
TrueModel.Nx = SIM.Nx;
TrueModel.Nd = SIM.Nd;
X = SIM.X;    

Nx = TrueModel.Nx;
Nwords = numel(X);

showInfo(Nm, Ns, LAG, BURN, Nest, Nx, Ny, Nz, T, SIM, TrueModel);

%% Setup results structure.
R.EST  = zeros([Ns, NEMD, Nm]);
R.pY_Z = zeros([Ns, Nm]);
R.pX_Y = zeros([Ns, Nm]);
R.pZ   = zeros([Ns, Nm]);
R.pXYZ = zeros([Ns, Nm]);
R.lpD_samp  = zeros([Ns, Nm]);
R.lpD_sampi = zeros([Nest, Nm]);
R.lpD_m_HM = zeros([Nest, Nm]);
R.lpD_m_DR = zeros([Nest, Nm]);

R.sZ_Xi     = zeros(T, Nest, Nm);
R.Rstat     = zeros([Ns, NEMD]);
R.Rrec.mean = zeros([NEMD, Nm]);
R.Rrec.disp = zeros([NEMD, Nm]);
R.topic_use = zeros([Ny, Nest, Nm]);         
R.clust_use = zeros([Nz, Nest, Nm]);
R.topic_use_m = zeros(Nest,Nm);
R.clust_use_m = zeros(Nest,Nm);

R.theta_est_s = zeros(Ny, Nz, Nest, Nm); 
R.chi_est_s   = zeros(Ny, T,  Nest, Nm); 
R.phi_est_s   = zeros(Nx, Ny, Nest, Nm);
R.BF_y        = zeros(ceil(Ns/LAG), Nm);
R.BF_z        = zeros(ceil(Ns/LAG), Nm);

if(TrueModel.INFER_HMM)
    if(SIM.REGULAR_HMM)
        R.psi_est_s   = zeros(Ny, Ny, Nest, Nm);        
    else
        R.psi_est_s   = zeros(Nz, Nz, Nest, Nm);
    end
else
    R.psi_est_s   = zeros(1, Nz, Nest, Nm);
end

%% Setup gibbs sampling structure.
BM = struct;
BM = repmat(BM, [1, Nm]);
for m = 1: Nm
    BM(m).sampZ_init = randi(Nz,[1,TrueModel.T]);
    BM(m).sampY_init = randi(Ny,[TrueModel.Nd,TrueModel.T]);
    BM(m).X=X;
    if(~INIT_TRUE || Ny~=TrueModel.Ny || Nz~=TrueModel.Nz)
        BM(m).sampY = BM(m).sampY_init;
        BM(m).sampZ = BM(m).sampZ_init;
        BM(m).lastY = BM(m).sampY_init;
        BM(m).lastZ = BM(m).sampZ_init;
    else 
        BM(m).sampY = Y; 
        BM(m).sampZ = Z;
        BM(m).lastY = Y; 
        BM(m).lastZ = Z;
    end
    if(SIM.USE_SSL)
        BM(m).sampZ(SIM.SSL.ZObin) = SIM.SSL.Z(SIM.SSL.ZObin);
    end
    BM(m).n_yz  = zeros(Ny, Nz);
    BM(m).n_xy  = zeros(Nx, Ny);
    BM(m).n_sxy = zeros(1,Ny);

    BM(m).n_sxy = zeros(1,Ny);
    BM(m).n_syy = zeros(1,Ny);
    BM(m).n_yy  = zeros(Ny,Ny);
    BM(m).n_sy  = 0;
    BM(m).n_y   = zeros(1,Ny);    
    
    BM(m).n_syz = zeros(1,Nz);
    BM(m).n_szz = zeros(1,Nz);
    BM(m).n_zz = zeros(Nz,Nz);
    BM(m).n_sz  = 0;
    BM(m).n_z   = zeros(1,Nz);
    
    BM(m).n_ym  = zeros(Ny, T);
    BM(m).n_sym   = zeros(1, T);
    BM(m) = util_recalculateStats_MCLDA(BM(m),TrueModel);
    
    [R.lpX_Y(1,m), R.lpY_Z(1,m), R.lpZ(1,m), R.lpXYZ(1,m)] = util_llh_MCLDA(BM(m), TrueModel);
    [BM(m).theta_est, BM(m).phi_est, BM(m).psi_est, BM(m).chi_est] = util_est_Params(BM(m),TrueModel);
    R.lpD_samp(1,m)= R.lpX_Y(1,m);
    
    R.EST(1,1,m) = R.lpXYZ(1,m);
    R.EST(1,2,m) = R.lpD_samp(1,m);
    R.EST(1,3,m) = R.lpX_Y(1,m);
    R.EST(1,4,m) = R.lpY_Z(1,m);
    R.EST(1,5,m) = R.lpZ(1,m);
end

[R.Rstat(1,:), R.Rrec.mean, R.Rrec.disp] = util_gelman_Rrec(1, R.Rrec, squeeze(R.EST(1,1:NEMD,:)));
if(DETAILS>=2), sfigure(1); clf; drawnow; end

ls = 0; lag_ctr = 1;

for i = 2 : Ns
    tic
    %% Run sample sweep for the appropriate model.
    if(REGULAR_LDA)
        %Vanilla LDA model.
        for m = 1 : Nm
            BM(m) = gibbs_sample_Bayes_LDA(BM(m),TrueModel);
        end
    end
    if(REGULAR_HMM)
        %Multi-observation HMM.
        parfor m = 1 : Nm
            BM(m) = gibbs_sample_Bayes_MO_HMM(BM(m),TrueModel);
        end
    end
    if(~REGULAR_HMM && ~REGULAR_LDA && INFER_HMM)
        %Our model: MCLDA
        parfor m = 1 : Nm
            BM(m) = gibbs_sample_Bayes_MCLDA_optimize(BM(m),TrueModel,SIM);                
        end
    end
    if(~REGULAR_HMM && ~REGULAR_LDA && ~INFER_HMM)
        %Simple mixture of topic models.
        parfor m = 1 : Nm
            BM(m) = gibbs_sample_Bayes_MixLDA(BM(m),TrueModel);
        end
    end
    
    m=1;
    for m = 1 : Nm
        [BM(m).theta_est, BM(m).phi_est, BM(m).psi_est, BM(m).chi_est] = util_est_Params(BM(m),TrueModel);
        
        [R.lpX_Y(i,m), R.lpY_Z(i,m), R.lpZ(i,m), R.lpXYZ(i,m)] = util_llh_MCLDA(BM(m), TrueModel);
        
        R.lpD_samp(i,m)= R.lpX_Y(i,m);

        R.EST(i,1,m) = R.lpXYZ(i,m);
        R.EST(i,2,m) = R.lpD_samp(i,m);
        R.EST(i,3,m) = R.lpX_Y(i,m);
        R.EST(i,4,m) = R.lpY_Z(i,m);
        R.EST(i,5,m) = R.lpZ(i,m);        
    end
    [R.Rstat(i,:), R.Rrec.mean, R.Rrec.disp] = util_gelman_Rrec(i, R.Rrec, squeeze(R.EST(i,1:NEMD,:)));
    
    %% Optionally record lots of details.
    if(DETAILS>=2 && (mod(i,10)-1==0))
        sfigure(2); clf;
        for m = 1 : 3
            subplot(3, 3, 3*(m-1)+1); imagesc(BM(m).theta_est); ylabel('Model m'); title('theta');
            subplot(3, 3, 3*(m-1)+2); imagesc(BM(m).phi_est); title('phi');
            subplot(3, 3, 3*(m-1)+3); imagesc(BM(m).psi_est); title('psi');
        end

        sfigure(1);
        subplot(2, 3, 1); cla; title('All Diagnostics, Sample Liks, Mean Dia, %Below 1.25'); hold on;
        plot(1:i,R.Rstat(1:i,1),'b-','linewidth',2.0);
        plot(1:i,R.Rstat(1:i,2),'g-','linewidth',2.0);
        plot(1:i,R.Rstat(1:i,3),'y-','linewidth',2.0);
        %plot(1:i,R.Rstat(1:i,5),'m-','linewidth',2.0);

        subplot(2, 3, 2); cla; title('pX|Y'); hold on;
        plot(R.lpX_Y(1:i,:), 'b:'); plot(1:i, mean(R.lpX_Y(1:i,:),2), 'b-');    

        subplot(2, 3, 3);cla; title('pY|Z'); hold on;
        plot(R.lpY_Z(1:i,:), 'g:'); plot(1:i, mean(R.lpY_Z(1:i,:),2), 'g-');    

        subplot(2, 3, 4);cla; hold on;
        plot(R.lpZ(1:i,:), 'r:');
        plot(1:i, mean(R.lpZ(1:i,:),2), 'r-');   title('Prior'); 

        subplot(2, 3, 5); cla; title('Joint Lik'); hold on;
        plot(R.lpXYZ(1:i,:), 'b:');
        plot(1:i, mean(R.lpXYZ(1:i,:),2), 'b-'); 
        
        if(COND>=3), sfigure(4); util_showImgC3(BM); end
        
%         if(i>BURN)
%             sfigure(3); clf; set(gcf,'Name','Most Recent Param Ests');
%             for m=1:3
%                 subplot(3, 3, 3*(m-1)+1); imagesc(R.theta_est_s(:,:,ls,m)); ylabel('Model m'); title('theta');
%                 subplot(3, 3, 3*(m-1)+2); imagesc(R.phi_est_s(:,:,ls,m)); title('phi');
%                 subplot(3, 3, 3*(m-1)+3); imagesc(R.psi_est_s(:,:,ls,m)); title('psi');
%             end
%         end
    end

    %% Save results.
    if(mod(i,LAG)-2==0)
        %After burn in, start computing model likelihood and estimates of        
        %parameters.
        %% Optional run until convergence.        
        %         if(ls==0)
        %             rm = mean(R.Rstat(i,:)); 
        %             if(SIM.WAIT_CONVERGE && rm>1.25)
        %                 fprintf(1,'Not   Logging: Mean RStat: %1.2f\n',rm);
        %                 ME = MException('gibbs_MC_LDA2:convergence','not converged');
        %                 throw(ME);
        %             else
        %                 fprintf(1,'Start Logging: Mean RStat: %1.2f\n',rm);
        %             end
        %         end
        %% Logging Params, LpD etc.    
        ls = ls + 1;
        nlpD_samp_m = zeros(ls, Nm);
        npD_samp_m = zeros(ls, Nm);
        lK_m = zeros(1, Nm);
        for j = 1 : Nm
            [R.theta_est_s(:,:,ls,j), R.phi_est_s(:,:,ls,j), R.psi_est_s(:,:,ls,j), R.chi_est_s(:,:,ls,j)] = util_est_Params(BM(j),TrueModel);

            R.lpD_sampi(ls,j) = R.lpX_Y(i,j);
            R.sY_Xi(:,:,ls,j) = BM(j).sampY;
            R.sZ_Xi(:,ls,j)   = BM(j).sampZ;
            
            lK_m(j) = max(R.lpD_sampi(1:ls,j), [], 1);
            nlpD_samp_m(:,j) = R.lpD_sampi(1:ls,j)-lK_m(j);
            npD_samp_m(:,j) = exp(nlpD_samp_m(:,j));
            R.lpD_m_HM(ls,j) = log(ls) + lK_m(j) - log(sum(1./npD_samp_m(:,j)));
            R.lpD_m_DR(ls,j) = lK_m(j) + log(sum(npD_samp_m(:,j)));
            %plot(1:ls, lpD_m(:,j), 'k-');
        end
        
        lK = max(max(R.lpD_sampi(1:ls,:)));
        nlpD_samp   = R.lpD_sampi(1:ls,:)-lK;
        npD_samp = exp(nlpD_samp);
        lpD(ls) = log(ls*Nm) + lK - log(sum(1./npD_samp(:)));

        %% Usage
        for m = 1 : Nm; 
            R.topic_use(:,ls,m) = R.topic_use(:,ls,m) + hist(BM(m).sampY(:),1:Ny)';
            R.topic_use_m(ls,m) = sum(R.topic_use(:,ls,m)>T/(Ny*20))/Ny;
        end
        for m = 1 : Nm; 
            R.clust_use(:,ls,m) = R.clust_use(:,ls,m) + hist(BM(m).sampZ(:),1:Nz)';
            R.clust_use_m(ls,m) = sum(R.clust_use(:,ls,m)>T/20)/Nz;
        end
        t = toc;        
        rw = max(R.Rstat(i,:)); 
        rm = mean(R.Rstat(i,:)); 
        fprintf(1, 'ETC: %d min. Worst/Mean Mix Ratio: %1.2f/%1.2f.', round((Ns-i)*t/60), rw, rm); 
        
        fprintf('  T Use: '); 
        for m = 1 : Nm, fprintf('%1.2f,', R.topic_use_m(ls,m)); end
        fprintf('.  C Use: '); 
        for m = 1 : Nm, fprintf('%1.2f,', R.clust_use_m(ls,m)); end               
        
        %% Logging bitflips.
        
        fprintf(1,' .Bitflips y/z: ');
        for j = 1 : m
            R.BF_y(lag_ctr,j) = R.BF_y(lag_ctr,j) + sum(BM(j).sampY(:) ~= BM(j).lastY(:));
            R.BF_z(lag_ctr,j) = R.BF_z(lag_ctr,j) + sum(BM(j).sampZ ~= BM(j).lastZ);
            BM(j).lastY = BM(j).sampY;
            BM(j).lastZ = BM(j).sampZ;
            fprintf(1, '%1.2f/%1.2f, ', R.BF_y(lag_ctr,j)/Nwords, R.BF_z(lag_ctr,j)/T);
        end
        fprintf(1,'\n');
        lag_ctr=lag_ctr+1;
        
        if(DETAILS>=2)
            sfigure(1);
            subplot(2, 3, 6); cla; hold on;
            plot(1:ls, lpD, 'b-'); title('estimated p(D) given samples so far');
        end
    end
    

%     if((mod(i,LAG)-2==0) && DETAILS>=0),  
%         rw = max(R.Rstat(i,:)); 
%         rm = mean(R.Rstat(i,:)); 
%         fprintf(1, 'ETC: %d min. Worst/Mean Mix Ratio: %1.2f/%1.2f\n', round((Ns-i)*t/60), rw, rm); 
%     end
    drawnow;
end

[~,R.hostname] = system('hostname');
R.date = datestr(now);
R.TrueModel = TrueModel;
R.BM = BM;
R.lpD = lpD;
disp('Done.');


function BM = gibbs_sample_Bayes_MixLDA(BM,TM)
Nx = TM.Nx;
[Ny,Nz] = size(BM.n_yz);

for t = 1 : TM.T
    z = BM.sampZ(t);
    for i = 1 : TM.Nd
        x = BM.X(i,t); 
        y = BM.sampY(i,t);
        BM.n_xy(x,y) = BM.n_xy(x,y) -1;
        BM.n_yz(y,z) = BM.n_yz(y,z) -1;
        BM.n_sxy(y)  = BM.n_sxy(y) - 1;
        BM.n_syz(z)  = BM.n_syz(z) - 1;
        
        pX_Y       = (BM.n_xy(x,:) +TM.beta )./(BM.n_sxy    + Nx*TM.beta);
        pY_ZYdiff  = (BM.n_yz(:,z)'+TM.alpha);%./(BM.n_syz(z) + Ny*TM.alpha);
        npY_XZYdiff =pX_Y .* pY_ZYdiff;
        pY_XZYdiff  = npY_XZYdiff./sum(npY_XZYdiff);
        
        %yn =  randsample(Ny,1,true,pY_XZYdiff);
        %yn =  sample(pY_XZYdiff,1);
        yn = sum(cumsum(pY_XZYdiff) < rand) + 1;
        
        BM.sampY(i,t) = yn;
        
        BM.n_xy(x,yn)  = BM.n_xy(x,yn) +1;
        BM.n_yz(yn,z)  = BM.n_yz(yn,z) +1;
        BM.n_sxy(yn)   = BM.n_sxy(yn)+1;
        BM.n_syz(z)   = BM.n_syz(z)+1;
    end

    % Sample the zs.
    %yhist = hist(BM.sampY(:,t),1:Ny);
    tmp =  histc(BM.sampY(:,t),1:Ny+1); yhist = tmp(1:Ny);
    n_yz_fdiff = BM.n_yz;
    n_yz_fdiff(:,z) = n_yz_fdiff(:,z) - yhist; %Unset this whole bunch of observations.
    BM.n_z(z) = BM.n_z(z)-1;
    lpY_Zdiff = zeros(1,Nz); 
    for zh = 1 : Nz
        n_yz_fplus = n_yz_fdiff(:,zh) + yhist; %Set observations for this hypothes.s
  
        lpY_Zdiff(zh)  = sum( gammaln( n_yz_fplus        + TM.alpha ) ) ...
                       - sum( gammaln( n_yz_fdiff(:,zh)  + TM.alpha ) ) ...
                       + gammaln ( sum( n_yz_fdiff(:,zh) + TM.alpha ) ) ...
                       - gammaln ( sum( n_yz_fplus       + TM.alpha ) );
    end
    pZ_Zdiff   = (BM.n_z+TM.gamma)./sum(BM.n_z+TM.gamma);
    lpZ_YZdiff = log(pZ_Zdiff) + lpY_Zdiff; 
    lkZ_Zdiff  = max(lpZ_YZdiff);
    pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
    %zn         = randsample(Nz,1,true,pZ_YZdiff);
    %zn         = sample(pZ_YZdiff,1);
    zn         = sum(cumsum(pZ_YZdiff) < rand) + 1;
    BM.sampZ(t)= zn;
    BM.n_z(zn) = BM.n_z(zn)+1;
    BM.n_yz = n_yz_fdiff;
    BM.n_yz(:,zn) = BM.n_yz(:,zn) + yhist;
end

function BM = gibbs_sample_Bayes_MO_HMM(BM,TM)
% Note the variables but not the distributions in this function have had their names changed.
% Externally, 'y' visible as sampZ.
[Nx,Ny] = size(BM.n_xy);
beta=TM.beta;
gamma=TM.gamma;

%% Z Resample: Time 1.
t=1;
y = BM.sampZ(t); 
y_new = BM.sampZ(t+1);              %Changed.
%yhist = hist(BM.sampY(:,t),1:Ny);  %Changed.
tmp =  histc(BM.X(:,t),1:Nx+1); xhist = tmp(1:Nx); %Changed.
n_xy_fdiff = BM.n_xy;               %Changed.

n_xy_fdiff(:,y) = n_xy_fdiff(:,y) - xhist; %Unset this whole bunch of observations.
lpY_Zdiff = zeros(1,Ny); 
BM.n_yy(y_new,y) = BM.n_yy(y_new,y) - 1;
BM.n_syy(y)      = BM.n_syy(y)-1;   %Num Transitions out of state y.

for yh = 1 : Ny
    n_xy_fplus = n_xy_fdiff(:,yh) + xhist; %Set observations for this hypothesis.
    lpY_Zdiff(yh)  = sum( gammaln( n_xy_fplus        + beta ) ) ...
                   - sum( gammaln( n_xy_fdiff(:,yh)  + beta) ) ...
                   + gammaln ( sum( n_xy_fdiff(:,yh) + beta ) ) ...
                   - gammaln ( sum( n_xy_fplus       + beta ) );
end
    
npZ_Zdiff  = (BM.n_yy(y_new,:)+gamma)./(BM.n_syy + Ny*gamma); %Vector num & denom: Every current y we could have transitioned out of to get to next y.
lpZ_YZdiff = log(npZ_Zdiff) + lpY_Zdiff;
lkZ_Zdiff  = max(lpZ_YZdiff);
pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
yn = sum(cumsum(pZ_YZdiff) < rand) + 1;
BM.sampZ(t)   = yn;

BM.n_xy       = n_xy_fdiff;
BM.n_xy(:,yn) = BM.n_xy(:,yn)+xhist;
BM.n_yy(y_new,yn) = BM.n_yy(y_new,yn) + 1;
BM.n_syy(yn)      = BM.n_syy(yn)+1;   %Num Transitions out of state y.

%% Time t
for t = 2 : TM.T - 1  
    % Resample the Zs.
    y     = BM.sampZ(t); y_old=BM.sampZ(t-1); y_new=BM.sampZ(t+1);
    
    %xhist = hist(BM.sampZ(:,t),1:Ny);
    tmp =  histc(BM.X(:,t),1:Nx+1); xhist = tmp(1:Nx);
    
    n_xy_fdiff = BM.n_xy;
    n_xy_fdiff(:,y)  = n_xy_fdiff(:,y) - xhist; %Unset this whole bunch of observations.
    BM.n_yy(y_new,y) = BM.n_yy(y_new,y)-1; %Transitions out of z.
    BM.n_yy(y,y_old) = BM.n_yy(y,y_old)-1; %Transitions into z.
	BM.n_syy(y)      = BM.n_syy(y)-1;
	BM.n_syy(y_old)  = BM.n_syy(y_old)-1;    
    I1 = (y_old==1:Ny); I2=I1.*(1:Ny==y_new);
    
    lpY_Zdiff = zeros(1,Ny); 
    for yh = 1 : Ny
        n_xy_fplus = n_xy_fdiff(:,yh) + xhist; %Set observations for this hypothesis.
  
        lpY_Zdiff(yh)  = sum( gammaln( n_xy_fplus        + beta ) ) ...
                       - sum( gammaln( n_xy_fdiff(:,yh)  + beta ) ) ...
                       + gammaln ( sum( n_xy_fdiff(:,yh) + beta ) ) ...
                       - gammaln ( sum( n_xy_fplus       + beta ) );
    end
    
%     %T1 is transitions into y, vector on output dimension 1. 
%     %T2 is transitions from y, vector on input dimnesion 2. 
    npZ_Zdiff = (BM.n_yy(:,y_old)'+gamma).*(BM.n_yy(y_new,:)+I2+gamma)./(BM.n_syy + I1 + Ny*gamma);
    %pZ_Zdiff  = (BM.n_z+TM.alpha)./sum(BM.n_z+TM.alpha);
    lpZ_YZdiff = log(npZ_Zdiff) + lpY_Zdiff; 
    lkZ_Zdiff  = max(lpZ_YZdiff);
    pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
    %yn         = randsample(Nz,1,true,pZ_YZdiff);
    %yn         =  sample(pZ_YZdiff,1);
    yn = sum(cumsum(pZ_YZdiff) < rand) + 1;
    BM.sampZ(t)= yn;

    BM.n_xy       = n_xy_fdiff;
    BM.n_xy(:,yn) = BM.n_xy(:,yn) + xhist;    
    BM.n_yy(y_new,yn) = BM.n_yy(y_new,yn)+1; %Transitions out of z.
    BM.n_yy(yn,y_old) = BM.n_yy(yn,y_old)+1; %Transitions into z.
    BM.n_syy(y_old)  = BM.n_syy(y_old)+1;    
	BM.n_syy(yn)      = BM.n_syy(yn)+1;
end

%% Z Resample: Time T.
t = t+1;
y = BM.sampZ(t); 
y_old = BM.sampZ(t-1);
%xhist = hist(BM.sampX(:,t),1:Nx);
tmp =  histc(BM.X(:,t),1:Nx+1); xhist = tmp(1:Nx);
n_xy_fdiff      = BM.n_xy;
n_xy_fdiff(:,y) = n_xy_fdiff(:,y) - xhist; %Unset this whole bunch of observations.
lpY_Zdiff       = zeros(1,Ny); 
BM.n_yy(y,y_old) = BM.n_yy(y,y_old) - 1;
BM.n_syy(y_old)  = BM.n_syy(y_old)-1;   %Num Transitions out of state y.

for yh = 1 : Ny
    n_xy_fplus = n_xy_fdiff(:,yh) + xhist; %Set observations for this hypothesis.
    lpY_Zdiff(yh)  = sum( gammaln( n_xy_fplus        + beta ) ) ...
                   - sum( gammaln( n_xy_fdiff(:,yh)  + beta) ) ...
                   + gammaln ( sum( n_xy_fdiff(:,yh) + beta ) ) ...
                   - gammaln ( sum( n_xy_fplus       + beta ) );
end
    
npZ_Zdiff  = (BM.n_yy(:,y_old)'+gamma);   %Vector num & denom: Every current y we could have transitioned out of to get to next y.
lpZ_YZdiff = log(npZ_Zdiff) + lpY_Zdiff;
lkZ_Zdiff  = max(lpZ_YZdiff);
pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
yn = sum(cumsum(pZ_YZdiff) < rand) + 1;
BM.sampZ(t)   = yn;

BM.n_xy       = n_xy_fdiff;
BM.n_xy(:,yn) = BM.n_xy(:,yn)+xhist;
BM.n_yy(yn,y_old) = BM.n_yy(yn,y_old) + 1;
BM.n_syy(y_old)  = BM.n_syy(y_old)+1;   %Num Transitions out of state y.

function BM = gibbs_sample_Bayes_MCLDA_optimize(BM,TM,SIM)
%% function BM = gibbs_sample_Bayes_MCLDA_optimize(BM,TM,SIM)
% Gibbs sampling for MCLDA: Optimized version.

Nx = TM.Nx;
[Ny,Nz] = size(BM.n_yz);
alpha=TM.alpha;
beta=TM.beta;
gamma=TM.gamma;
Nx_beta = Nx*TM.beta;
n_xy  = BM.n_xy + beta;
n_sxy = BM.n_sxy + Nx_beta;
n_yz  = BM.n_yz + alpha;
%% Resample the Topics Y.
for t = 1 : TM.T
    z = BM.sampZ(t);
    for i = 1 : TM.Nd
        x = BM.X(i,t); 
        y = BM.sampY(i,t);
        n_xy(x,y) = n_xy(x,y) -1;
        n_yz(y,z) = n_yz(y,z) -1;
        n_sxy(y)  = n_sxy(y) - 1;    

        %Increasingly efficient probability dist calculations.
        %pX_Y       = (BM.n_xy(x,:) +TM.beta )./(BM.n_sxy + Nx_beta);
        %pY_ZYdiff  = (BM.n_yz(:,z)'+TM.alpha);%propto./(BM.n_syz(z) + Ny*TM.alpha);
        %npY_XZYdiff = pX_Y .* pY_ZYdiff;
        npY_XZYdiff = n_xy(x,:) .* n_yz(:,z)'./n_sxy;
        pY_XZYdiff  = npY_XZYdiff./sum(npY_XZYdiff);
        
        %Increasingly efficient random samplers.
        %yn =  randsample(Ny,1,true,pY_XZYdiff);
        %yn =  sample(pY_XZYdiff,1);
        yn = sum(cumsum(pY_XZYdiff) < rand) + 1;
        
        BM.sampY(i,t) = yn;
        
        n_xy(x,yn)  = n_xy(x,yn) +1;
        n_yz(yn,z)  = n_yz(yn,z) +1;
        n_sxy(yn)   = n_sxy(yn)+1;
    end
end
BM.n_xy = n_xy - beta;
BM.n_sxy = n_sxy - Nx_beta;
BM.n_yz = n_yz - alpha;

%% Resample the clusters, Z. Time 1.
t=1;
if(~SIM.USE_SSL || (SIM.USE_SSL&&~SIM.SSL.ZObin(t)))
    z = BM.sampZ(t); 
    z_new = BM.sampZ(t+1);    
    %yhist = hist(BM.sampY(:,t),1:Ny);
    tmp =  histc(BM.sampY(:,t),1:Ny+1); yhist = tmp(1:Ny);
    n_yz_fdiff = BM.n_yz;
    n_yz_fdiff(:,z) = n_yz_fdiff(:,z) - yhist; %Unset this whole bunch of observations.
    lpY_Zdiff = zeros(1,Nz); 
    BM.n_zz(z_new,z) = BM.n_zz(z_new,z) - 1;
    BM.n_szz(z)      = BM.n_szz(z)-1;   %Num Transitions out of state y.

    for zh = 1 : Nz
        n_yz_fplus = n_yz_fdiff(:,zh) + yhist; %Set observations for this hypothesis.
        lpY_Zdiff(zh)  = sum( gammaln( n_yz_fplus        + alpha ) ) ...
                       - sum( gammaln( n_yz_fdiff(:,zh)  + alpha) ) ...
                       + gammaln ( sum( n_yz_fdiff(:,zh) + alpha ) ) ...
                       - gammaln ( sum( n_yz_fplus       + alpha ) );
    end

    npZ_Zdiff  = (BM.n_zz(z_new,:)+gamma)./(BM.n_szz + Nz*gamma); %Vector num & denom: Every current y we could have transitioned out of to get to next y.
    lpZ_YZdiff = log(npZ_Zdiff) + lpY_Zdiff;
    lkZ_Zdiff  = max(lpZ_YZdiff);
    pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
    %zn         = randsample(Nz,1,true,pZ_YZdiff);
    %zn         =  sample(pZ_YZdiff,1);
    zn = sum(cumsum(pZ_YZdiff) < rand) + 1;
    BM.sampZ(t)   = zn;

    BM.n_yz       = n_yz_fdiff;
    BM.n_yz(:,zn) = BM.n_yz(:,zn)+yhist;
    BM.n_zz(z_new,zn) = BM.n_zz(z_new,zn) + 1;
    BM.n_szz(zn)      = BM.n_szz(zn)+1;   %Num Transitions out of state y.
end

%% Resample the clusters, Z. Time t.
for t = 2 : TM.T - 1  
    if(~SIM.USE_SSL || (SIM.USE_SSL&&~SIM.SSL.ZObin(t)))
        % Resample the Zs.
        z     = BM.sampZ(t); z_old=BM.sampZ(t-1); z_new=BM.sampZ(t+1);

        %yhist = hist(BM.sampY(:,t),1:Ny);
        tmp =  histc(BM.sampY(:,t),1:Ny+1); yhist = tmp(1:Ny);

        n_yz_fdiff = BM.n_yz;
        n_yz_fdiff(:,z)  = n_yz_fdiff(:,z) - yhist; %Unset this whole bunch of observations.
        BM.n_zz(z_new,z) = BM.n_zz(z_new,z)-1; %Transitions out of z.
        BM.n_zz(z,z_old) = BM.n_zz(z,z_old)-1; %Transitions into z.
        BM.n_szz(z)      = BM.n_szz(z)-1;
        BM.n_szz(z_old)  = BM.n_szz(z_old)-1;    
        I1 = (z_old==1:Nz); I2=I1.*(1:Nz==z_new);

        lpY_Zdiff = zeros(1,Nz); 
        for zh = 1 : Nz
            n_yz_fplus = n_yz_fdiff(:,zh) + yhist; %Set observations for this hypothesis.

            lpY_Zdiff(zh)  = sum( gammaln( n_yz_fplus        + alpha ) ) ...
                           - sum( gammaln( n_yz_fdiff(:,zh)  + alpha) ) ...
                           + gammaln ( sum( n_yz_fdiff(:,zh) + alpha ) ) ...
                           - gammaln ( sum( n_yz_fplus       + alpha ) );
        end

    %     %T1 is transitions into y, vector on output dimension 1. 
    %     %T2 is transitions from y, vector on input dimnesion 2. 
        npZ_Zdiff = (BM.n_zz(:,z_old)'+gamma).*(BM.n_zz(z_new,:)+I2+gamma)./(BM.n_szz + I1 + Nz*gamma);
        %pZ_Zdiff  = (BM.n_z+TM.alpha)./sum(BM.n_z+TM.alpha);
        lpZ_YZdiff = log(npZ_Zdiff) + lpY_Zdiff; 
        lkZ_Zdiff  = max(lpZ_YZdiff);
        pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
        %zn         = randsample(Nz,1,true,pZ_YZdiff);
        %zn         =  sample(pZ_YZdiff,1);
        zn = sum(cumsum(pZ_YZdiff) < rand) + 1;
        BM.sampZ(t)= zn;

        BM.n_yz       = n_yz_fdiff;
        BM.n_yz(:,zn) = BM.n_yz(:,zn) + yhist;    
        BM.n_zz(z_new,zn) = BM.n_zz(z_new,zn)+1; %Transitions out of z.
        BM.n_zz(zn,z_old) = BM.n_zz(zn,z_old)+1; %Transitions into z.
        BM.n_szz(z_old)  = BM.n_szz(z_old)+1;    
        BM.n_szz(zn)      = BM.n_szz(zn)+1;
    end
end

%% Resample the clusters, Z. Time T.
t = t+1;
if(~SIM.USE_SSL || (SIM.USE_SSL&&~SIM.SSL.ZObin(t)))
    z = BM.sampZ(t); 
    z_old = BM.sampZ(t-1);
    %yhist = hist(BM.sampY(:,t),1:Ny);
    tmp =  histc(BM.sampY(:,t),1:Ny+1); yhist = tmp(1:Ny);
    n_yz_fdiff      = BM.n_yz;
    n_yz_fdiff(:,z) = n_yz_fdiff(:,z) - yhist; %Unset this whole bunch of observations.
    lpY_Zdiff       = zeros(1,Nz); 
    BM.n_zz(z,z_old) = BM.n_zz(z,z_old) - 1;
    BM.n_szz(z_old)  = BM.n_szz(z_old)-1;   %Num Transitions out of state y.

    for zh = 1 : Nz
        n_yz_fplus = n_yz_fdiff(:,zh) + yhist; %Set observations for this hypothesis.
        lpY_Zdiff(zh)  = sum( gammaln( n_yz_fplus        + alpha ) ) ...
                       - sum( gammaln( n_yz_fdiff(:,zh)  + alpha) ) ...
                       + gammaln ( sum( n_yz_fdiff(:,zh) + alpha ) ) ...
                       - gammaln ( sum( n_yz_fplus       + alpha ) );
    end

    npZ_Zdiff  = (BM.n_zz(:,z_old)'+gamma);   %Vector num & denom: Every current y we could have transitioned out of to get to next y.
    lpZ_YZdiff = log(npZ_Zdiff) + lpY_Zdiff;
    lkZ_Zdiff  = max(lpZ_YZdiff);
    pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
    %zn         = randsample(Nz,1,true,pZ_YZdiff);
    %zn         =  sample(pZ_YZdiff,1);
    zn = sum(cumsum(pZ_YZdiff) < rand) + 1;
    BM.sampZ(t)   = zn;

    BM.n_yz       = n_yz_fdiff;
    BM.n_yz(:,zn) = BM.n_yz(:,zn)+yhist;
    BM.n_zz(zn,z_old) = BM.n_zz(zn,z_old) + 1;
    BM.n_szz(z_old)  = BM.n_szz(z_old)+1;   %Num Transitions out of state y.
end

function BM = gibbs_sample_Bayes_MCLDA(BM,TM)
% Gibbs sampling for MCLDA: Simple version.
Nx = TM.Nx;
[Ny,Nz] = size(BM.n_yz);
alpha=TM.alpha;
gamma=TM.gamma;
Nx_beta = Nx*TM.beta;
%% Resample the Ys.
for t = 1 : TM.T
    z = BM.sampZ(t);
    for i = 1 : TM.Nd
        x = BM.X(i,t); 
        y = BM.sampY(i,t);
        BM.n_xy(x,y) = BM.n_xy(x,y) -1;
        BM.n_yz(y,z) = BM.n_yz(y,z) -1;
        BM.n_sxy(y)  = BM.n_sxy(y) - 1;
        %BM.n_syz(z)  = BM.n_syz(z) - 1;
        
        pX_Y       = (BM.n_xy(x,:) +TM.beta )./(BM.n_sxy + Nx_beta);
        pY_ZYdiff  = (BM.n_yz(:,z)'+TM.alpha);%propto./(BM.n_syz(z) + Ny*TM.alpha);
        %pY_XZYdiff = normalise(pX_Y .* pY_ZYdiff);
        npY_XZYdiff = pX_Y .* pY_ZYdiff;
        pY_XZYdiff  = npY_XZYdiff./sum(npY_XZYdiff);
        %yn =  randsample(Ny,1,true,pY_XZYdiff);
        %yn =  sample(pY_XZYdiff,1);
        yn = sum(cumsum(pY_XZYdiff) < rand) + 1;
        
        BM.sampY(i,t) = yn;
        
        BM.n_xy(x,yn)  = BM.n_xy(x,yn) +1;
        BM.n_yz(yn,z)  = BM.n_yz(yn,z) +1;
        BM.n_sxy(yn)   = BM.n_sxy(yn)+1;
        %BM.n_syz(z)   = BM.n_syz(z)+1;
    end
end

%% Z Resample: Time 1.
t=1;
z = BM.sampZ(t); 
z_new = BM.sampZ(t+1);    
%yhist = hist(BM.sampY(:,t),1:Ny);
tmp =  histc(BM.sampY(:,t),1:Ny+1); yhist = tmp(1:Ny);
n_yz_fdiff = BM.n_yz;
n_yz_fdiff(:,z) = n_yz_fdiff(:,z) - yhist; %Unset this whole bunch of observations.
lpY_Zdiff = zeros(1,Nz); 
BM.n_zz(z_new,z) = BM.n_zz(z_new,z) - 1;
BM.n_szz(z)      = BM.n_szz(z)-1;   %Num Transitions out of state y.

for zh = 1 : Nz
    n_yz_fplus = n_yz_fdiff(:,zh) + yhist; %Set observations for this hypothesis.
    lpY_Zdiff(zh)  = sum( gammaln( n_yz_fplus        + alpha ) ) ...
                   - sum( gammaln( n_yz_fdiff(:,zh)  + alpha) ) ...
                   + gammaln ( sum( n_yz_fdiff(:,zh) + alpha ) ) ...
                   - gammaln ( sum( n_yz_fplus       + alpha ) );
end
    
npZ_Zdiff  = (BM.n_zz(z_new,:)+gamma)./(BM.n_szz + Nz*gamma); %Vector num & denom: Every current y we could have transitioned out of to get to next y.
lpZ_YZdiff = log(npZ_Zdiff) + lpY_Zdiff;
lkZ_Zdiff  = max(lpZ_YZdiff);
pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
%zn         = randsample(Nz,1,true,pZ_YZdiff);
%zn         =  sample(pZ_YZdiff,1);
zn = sum(cumsum(pZ_YZdiff) < rand) + 1;
BM.sampZ(t)   = zn;

BM.n_yz       = n_yz_fdiff;
BM.n_yz(:,zn) = BM.n_yz(:,zn)+yhist;
BM.n_zz(z_new,zn) = BM.n_zz(z_new,zn) + 1;
BM.n_szz(zn)      = BM.n_szz(zn)+1;   %Num Transitions out of state y.

%% Time t
for t = 2 : TM.T - 1  
    % Resample the Zs.
    z     = BM.sampZ(t); z_old=BM.sampZ(t-1); z_new=BM.sampZ(t+1);
    
    %yhist = hist(BM.sampY(:,t),1:Ny);
    tmp =  histc(BM.sampY(:,t),1:Ny+1); yhist = tmp(1:Ny);
    
    n_yz_fdiff = BM.n_yz;
    n_yz_fdiff(:,z)  = n_yz_fdiff(:,z) - yhist; %Unset this whole bunch of observations.
    BM.n_zz(z_new,z) = BM.n_zz(z_new,z)-1; %Transitions out of z.
    BM.n_zz(z,z_old) = BM.n_zz(z,z_old)-1; %Transitions into z.
	BM.n_szz(z)      = BM.n_szz(z)-1;
	BM.n_szz(z_old)  = BM.n_szz(z_old)-1;    
    I1 = (z_old==1:Nz); I2=I1.*(1:Nz==z_new);
    
    lpY_Zdiff = zeros(1,Nz); 
    for zh = 1 : Nz
        n_yz_fplus = n_yz_fdiff(:,zh) + yhist; %Set observations for this hypothesis.
  
        lpY_Zdiff(zh)  = sum( gammaln( n_yz_fplus        + alpha ) ) ...
                       - sum( gammaln( n_yz_fdiff(:,zh)  + alpha) ) ...
                       + gammaln ( sum( n_yz_fdiff(:,zh) + alpha ) ) ...
                       - gammaln ( sum( n_yz_fplus       + alpha ) );
    end
    
%     %T1 is transitions into y, vector on output dimension 1. 
%     %T2 is transitions from y, vector on input dimnesion 2. 
    npZ_Zdiff = (BM.n_zz(:,z_old)'+gamma).*(BM.n_zz(z_new,:)+I2+gamma)./(BM.n_szz + I1 + Nz*gamma);
    %pZ_Zdiff  = (BM.n_z+TM.alpha)./sum(BM.n_z+TM.alpha);
    lpZ_YZdiff = log(npZ_Zdiff) + lpY_Zdiff; 
    lkZ_Zdiff  = max(lpZ_YZdiff);
    pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
    %zn         = randsample(Nz,1,true,pZ_YZdiff);
    %zn         =  sample(pZ_YZdiff,1);
    zn = sum(cumsum(pZ_YZdiff) < rand) + 1;
    BM.sampZ(t)= zn;

    BM.n_yz       = n_yz_fdiff;
    BM.n_yz(:,zn) = BM.n_yz(:,zn) + yhist;    
    BM.n_zz(z_new,zn) = BM.n_zz(z_new,zn)+1; %Transitions out of z.
    BM.n_zz(zn,z_old) = BM.n_zz(zn,z_old)+1; %Transitions into z.
    BM.n_szz(z_old)  = BM.n_szz(z_old)+1;    
	BM.n_szz(zn)      = BM.n_szz(zn)+1;
end

%% Z Resample: Time T.
t = t+1;
z = BM.sampZ(t); 
z_old = BM.sampZ(t-1);
%yhist = hist(BM.sampY(:,t),1:Ny);
tmp =  histc(BM.sampY(:,t),1:Ny+1); yhist = tmp(1:Ny);
n_yz_fdiff      = BM.n_yz;
n_yz_fdiff(:,z) = n_yz_fdiff(:,z) - yhist; %Unset this whole bunch of observations.
lpY_Zdiff       = zeros(1,Nz); 
BM.n_zz(z,z_old) = BM.n_zz(z,z_old) - 1;
BM.n_szz(z_old)  = BM.n_szz(z_old)-1;   %Num Transitions out of state y.

for zh = 1 : Nz
    n_yz_fplus = n_yz_fdiff(:,zh) + yhist; %Set observations for this hypothesis.
    lpY_Zdiff(zh)  = sum( gammaln( n_yz_fplus        + alpha ) ) ...
                   - sum( gammaln( n_yz_fdiff(:,zh)  + alpha) ) ...
                   + gammaln ( sum( n_yz_fdiff(:,zh) + alpha ) ) ...
                   - gammaln ( sum( n_yz_fplus       + alpha ) );
end
    
npZ_Zdiff  = (BM.n_zz(:,z_old)'+gamma);   %Vector num & denom: Every current y we could have transitioned out of to get to next y.
lpZ_YZdiff = log(npZ_Zdiff) + lpY_Zdiff;
lkZ_Zdiff  = max(lpZ_YZdiff);
pZ_YZdiff  = normalise(exp(lpZ_YZdiff-lkZ_Zdiff));
%zn         = randsample(Nz,1,true,pZ_YZdiff);
%zn         =  sample(pZ_YZdiff,1);
zn = sum(cumsum(pZ_YZdiff) < rand) + 1;
BM.sampZ(t)   = zn;

BM.n_yz       = n_yz_fdiff;
BM.n_yz(:,zn) = BM.n_yz(:,zn)+yhist;
BM.n_zz(zn,z_old) = BM.n_zz(zn,z_old) + 1;
BM.n_szz(z_old)  = BM.n_szz(z_old)+1;   %Num Transitions out of state y.
  
function BM=gibbs_sample_Bayes_LDA(BM,TM)
%% function BM=gibbs_sample_Bayes_LDA(BM,TM)
% Gibbs sampling for LDA.
% TODO: Optimze this by moving the +alpha, beta out of the loop like MCLDA.
T = TM.T; Nd = TM.Nd;
alpha = TM.alpha;
beta = TM.beta;
[Nx,~]=size(BM.n_xy);

for t = 1 : T
    %BM.n_m(t) = BM.n_m(t)-1;
    for i = 1 : Nd
        x = BM.X(i,t);
        y = BM.sampY(i,t);

        %Decrement.
        BM.n_sym(t)  = BM.n_sym(t)-1;            
        BM.n_sxy(y)  = BM.n_sxy(y)-1;            
        BM.n_xy(x,y) = BM.n_xy(x,y)-1;    %Word-Topic assignment.
        BM.n_ym(y,t) = BM.n_ym(y,t)-1;    %Topic-Doc assignment.

        %Compute p(y_t|Y\t,X)
        %py_YX = ((BM.n_xy(x,:) + beta)./(BM.n_sxy + Nx*beta)).*((BM.n_ym(:,t)' + alpha)/(BM.n_sym(t) + Ny*alpha));
        py_YX = ((BM.n_xy(x,:) + beta)./(BM.n_sxy + Nx*beta)).*(BM.n_ym(:,t)' + alpha); %propto above %/(BM.n_sym(t) + Ny*alpha));

        %pz_ZW = normalise(pz_ZW);
        py_YX = py_YX/sum(py_YX);
        
        %Increasingly efficient series of random sampling schemes.
        %yn = discrete_sample_rejection(py_YX,1);
        %yn =  randsample(Ny,1,true,py_YX);
        %yn =  sample(py_YX,1);
        yn = sum(cumsum(py_YX) < rand) + 1;
        
        BM.sampY(i,t) = yn;
        
        %Increment.
        %BM.n_y(yn) = BM.n_y(yn)+1;   
        BM.n_sym(t)   = BM.n_sym(t)+1;            
        BM.n_sxy(yn)  = BM.n_sxy(yn)+1;                    
        BM.n_xy(x,yn) = BM.n_xy(x,yn)+1;    %Word-Topic assignment.
        BM.n_ym(yn,t) = BM.n_ym(yn,t)+1;    %Topic-Doc assignment.          
    end
    %BM.n_m(t) = BM.n_m(t)+1;
end

function showInfo(Nm, Ns, LAG, BURN, NEst, Nx, Ny, Nz, T, SIM, TM)
fprintf(1,'gibbs_MC_LDA1: Nm: %d, Ns: %d, LAG: %d, BURN: %d, NEst: %d, INIT_TRUE: %d, Time: %d\n\tNx: %d. Assumed (Ny: %d, Nz: %d).', ...
    Nm, Ns, LAG, BURN, NEst, TM.INIT_TRUE, T, Nx, Ny, Nz);
if(nargin>10)
    fprintf(1,'True: (Ny: %d, Nz: %d)\n', TM.Ny, TM.Nz);
else
    fprintf(1,'True: Unknown.');
end
fprintf(1,'\tHyperpriors: Alpha: %1.1f, Beta: %1.1f, Gamma: %1.1f\n', TM.alpha, TM.beta, TM.gamma);
fprintf(1,'\tREGULAR LDA: %d, Infer HMM: %d. Wait Converge: %d \n', TM.REGULAR_LDA, TM.INFER_HMM, SIM.WAIT_CONVERGE);
if(SIM.USE_SSL)
    fprintf(1,'\tSSL: Yes. %d/%d observed\n',SIM.SSL.Nobs,T);
else
    fprintf(1,'\tSSL: Off\n');
end
    
function nX = normalise(X)
%function nX = normalise(X)
% Quicker normalise.
s = sum(X(:));
nX = X/s;

